	LIST P=16f628
      ERRORLEVEL  -302
;
; This program interfaces to a Hitachi compatible 2 line by 20 character display
; module. The program assembles for 4-bit data serial interface through shift
; register (74HC164) by 3 wire. Shift register CD4094 may also can be used but need 4 wire. 
; The data, commond and RS line for LCD was control by MCU through shift register. 
; Only Enable line for LCD was directly control by MCU.
;
;       Program = 628S_L.ASM
;       Revision Date:   10-08-2001
;
;
       include <p16f628.inc>
	page
	__CONFIG _CP_OFF & _XT_OSC & _PWRTE_ON & _WDT_OFF
	
;**************************************************************************************
;Definition and address/register setting for temporary data storage. 
;*************************************************************************************
Dev_Freq		EQU		D'4000000'			; define Device Frequency is 4 MHz
DB_HI_BYTE		EQU		(HIGH ((( Dev_Freq / 4 ) * 1 / D'1000' ) / 2 ) ) + 1
LCD_INIT_DELAY	EQU		(HIGH ((( Dev_Freq / 4 ) * D'66' / D'10000' ) / 2 ) ) + 1
;
DISP_ON		EQU		0x00e			;define Display on value
CLR_DISP		EQU		0x001		
ENTRY_INC		EQU		0x028			;define 4 bit mode for LCD display
CHAR			EQU		0x020			;address for temporary store value will 
								;be send to LCD through shift register
TEMP			EQU		0x021	
MSD			EQU		0x022			;Temporary register, Holds Most Significant Digit of BIN to BCD conversion
LSD			EQU		0x023			;Temporary register
TEMP1			EQU		0x024			;Temporary register 
repeat		equ		0x025			;Temporary register
;
dly5		equ	0x026    
;
serial_port 	equ	PORTA	;set serial I/O port register
DO		equ	0		;define serial data output pin from MCU
SCK		equ	1		;define clock output pin from MCU
STR		equ	2		;define store pin output from MCU

bit_count	equ	0x27		;Temporary register
rxdata	equ	0x28		;Temporary register
data_out	equ	0x29		;Temporary register
data_temp	equ	0x2a		;Temporary register

Dlybyte2	equ	0x2b		;Temporary register
Dlybyte1	equ	0x2c		;Temporary register

addrMSB0	equ	0x2d		;Temporary register
addrLSB0	equ	0x2e		;Temporary register
;
; LCD Display Commands and Control Signal names.
;
LCD_CNTL    EQU   PORTA		;set LCD control register
E		EQU	3		;LCD Enable control line
;------------------------------------------------------------------
;-spi port definition
;------------------------------------------------------------------
spi_port1 	equ	PORTB
DI1		equ	6
SCK1		equ	7
CS1		equ	3
DO1		equ	5
bit_count1	equ	0x2f

rxdata1	equ	0x30
data_out1	equ	0x31
data_temp1	equ	0x32

adcMSB_temp	equ	0x33		;temperary address to store the ADC binary code data
adcLSB_temp	equ	0x34

ch_LSB_temp	equ	0x35		;temperary address to store the start up setting
ch_MSB_temp	equ	0x36		;for ADC conversion code and ADC channel selection

dlybyte	equ	0x37

ADC_temp1	equ	0x38		;temp address for binary convertsion
ADC_temp2	equ	0x39
ADC_LSB_H	equ	0x3a
ADC_MSB_H	equ	0x3b		;temp address for binary convertsion

ADC_F_LSB	equ	0x3c		;temp address for proces binary convertsion
ADC_S_LSB	equ	0x3d
ADC_F_MSB	equ	0x3e
ADC_S_MSB	equ	0x3f		;temp address for proces binary convertsion

W_SAVE	EQU	0X40
STATUS_SAVE	EQU	0x41
STORE_BIT0	EQU	0x42

int_dly	equ	0x43
int_dly0	equ	0x44	
int_dly1	equ	0x45

;**********************************************************************************
;Begining of program
;**********************************************************************************
       org     	0x0000		; RESET vector location
       GOTO    	START          ;
	ORG		0X004
	GOTO		INT_SERV
;
;
START 
	call		Port_Init
	call		LCD_DELAY
	call		CMD_SEQ		;sequence to initializa the LCD
	call		Display_data0
	movlw		0x01
	movwf		STORE_BIT0
check	
	call		precess_chennal
	call		process_ADC_display
	bsf		OPTION_REG, INTEDG
clear_int1
	BcF		INTCON, INTF
	btfsc		INTCON, INTF
	goto		clear_int1
	BSF		INTCON, INTE
	BSF		INTCON, GIE
	goto		check
  	

;----------------------------------------------------------------------------
;SUB: INTERRUPT VECTOR
;----------------------------------------------------------------------------
INT_SERV
	MOVWF		W_SAVE
	SWAPF		STATUS, W
	MOVWF		STATUS_SAVE
	BCF 		STATUS, RP0
	movf		STORE_BIT0, w
	ANDLW 	0XF8
	btfsc		STATUS, Z
	GOTO		INC_BIT0
	GOTO		CLEAR_BIT0
INC_BIT0
	call		int_delay
	INCF		STORE_BIT0, F	
	BCF		INTCON, INTF
	SWAPF		STATUS_SAVE, W
	MOVWF		STATUS
	SWAPF		W_SAVE, F
	SWAPF		W_SAVE, W
	RETFIE
CLEAR_BIT0
	call		int_delay
	CLRF		STORE_BIT0
	movlw		0x01
	movwf		STORE_BIT0
	BCF		INTCON, INTF
	SWAPF		STATUS_SAVE, W
	MOVWF		STATUS
	SWAPF		W_SAVE, F
	SWAPF		W_SAVE, W
	RETFIE

int_delay
	movlw	0x02
	movwf	int_dly1
dlyloop3
	movlw	0xf0
	movwf	int_dly0
dlyloop2
	movlw	0xff
	movwf	int_dly
dlyloop1
	decfsz	int_dly,1
	goto		dlyloop1
	decfsz	int_dly0,1
	goto		dlyloop2
	decfsz	int_dly1,1
	goto		dlyloop3
	return


;-----------------------------------------------------------------------------
;sub: process adc data conversion and display on LCD
;-----------------------------------------------------------------------------	
process_ADC_display
	movlw   	B'11001111'     ;Address DDRam first character, first line
      call    	SEND_CMD
	movf		ADC_F_LSB, w
	call		SEND_CHAR
	movlw   	B'11001110'     ;Address DDRam first character, first line
      call    	SEND_CMD
	movf		ADC_S_LSB, w
	call		SEND_CHAR
	movlw   	B'11001101'     ;Address DDRam first character, first line
      call    	SEND_CMD
	movf		ADC_F_MSB, w
	call		SEND_CHAR
	movlw   	B'11001100'     ;Address DDRam first character, first line
      call    	SEND_CMD
	movf		ADC_S_MSB, w
	call		SEND_CHAR
	call		delay
	return
	
   
;************************************************************************************
;Initialization of hardware and Port setting
;************************************************************************************
Port_Init
      CLRF    	STATUS          ; Do initialization (Bank 0)
      bcf	INTCON,7
      clrf	PORTA
	clrf	PORTB
	movlw	0x07
	movwf	CMCON            
;
      BSF     	STATUS, RP0     ; Select Bank 1
	movlw	b'01111111'
	movwf	OPTION_REG
      CLRF    	TRISA           ; RA5 -  0 outputs
      movlw	  	b'01000011'
	movwf	  	TRISB			; RB7, 5 - 2 output, RB6, 1 - 0 input            
      BCF     	STATUS, RP0     ; Select Bank 0
	return


;******************************************************************************
; This routine takes the calculated times that the delay loop needs to
; be executed, based on the LCD_INIT_DELAY EQUate that includes the
; frequency of operation. These uses registers before they are needed to 
; store the time.
;*******************************************************************************
LCD_DELAY   
	MOVLW   	LCD_INIT_DELAY  ;
      MOVWF   	MSD             ; Use MSD and LSD Registers to Initilize LCD
      CLRF    	LSD             ;
LOOP2 
      DECFSZ  	LSD, F          ; Delay time = MSD * ((3 * 256) + 3) * Tcy
      GOTO    	LOOP2           ;
      DECFSZ  	MSD, F          ;
END_LCD_DELAY
      GOTO    	LOOP2           ;
	return

;***********************************************************************************
; Command sequence for 2 lines of 5x7 characters LCD modules
;***********************************************************************************
CMD_SEQ
	movlw		0x03
	movwf		repeat
reloop
	MOVLW 	0x28       		;8-bit high nibble xfer
      movwf		data_out
	call		serial		;output command through shift register
	call		delay_interval
	BSF   	LCD_CNTL, E     	;
	nop
      BCF   	LCD_CNTL, E     	;
	decfsz	repeat, 1
	goto		reloop

	movlw		0x16
	movwf		dly5
loopdly
	DECFSZ 	dly5, F     ; Delay time = MSD * ((3 * 256) + 3) * Tcy
      GOTO    	loopdly           
		

      MOVLW   	0x06          ; 8-bit high nibble xfer
      CALL    	SEND_CMD 
      return

;******************************************************************************
;Sub : serial
;Description : shift register serial interfacing
;******************************************************************************
serial
	movlw		h'08'
	movwf		bit_count
	bcf		serial_port, DO
	bcf		serial_port, STR
rwspiloop
	movf		data_out,w
	andlw		0x80
	movwf		data_temp
	bcf		serial_port,SCK	;start here, sck = 0
	btfss		data_temp,7		;check bit 7 equ to 1?
	bcf		serial_port,DO	;no, DO = 0
	btfsc		data_temp,7		;No, check bit 7 equ to 0?
	bsf		serial_port,DO	;no, DO = 1
	bsf		serial_port, STR	;storage register clock high
	bsf		serial_port,SCK	;sck = 1
	call		dlyus
	bcf		serial_port, STR	;storage register clock low
	rlf		data_out,7
	decfsz	bit_count,1
	goto		rwspiloop
	bcf		serial_port,SCK
	bcf		serial_port, STR	;storage register clock low
	bcf		serial_port, DO
	nop
	nop
	return


;*******************************************************************
;*SendChar - Sends character to LCD                                *
;*This routine splits the character into the upper and lower       * 
;*nibbles and sends them to the LCD, upper nibble first.           *
;*******************************************************************
SEND_CHAR
      movwf		CHAR
	MOVF    	CHAR,w
      ANDLW   	0xF0            ;Get upper nibble
	iorlw		0x01			;RS high (LCD)
	movwf		data_out
      call		serial		;Send data to LCD through shift register
      CALL    	delay_interval  ;Wait for LCD to be ready
      BSF     	LCD_CNTL, E     ;toggle E for LCD
	nop
      BCF     	LCD_CNTL, E
      SWAPF   	CHAR, w
      ANDLW   	0xF0            ;Get lower nibble
	iorlw		0x01
	movwf		data_out
      call		serial		;Send data to LCD
      BSF     	LCD_CNTL, E     ;toggle E for LCD
	nop
      BCF     	LCD_CNTL, E
      RETURN

;*******************************************************************
;* SEND_CMD - Sends command to LCD                                 *
;* This routine splits the command into the upper and lower        * 
;* nibbles and sends them to the LCD, upper nibble first.          *
;*******************************************************************
SEND_CMD
      movwf		CHAR
	MOVF    	CHAR,w
      ANDLW   	0xF0            ;Get upper nibble
	movwf		data_out
      call		serial		;Send data to LCD through shift register
	CALL    	delay_interval  ;Wait for LCD to be ready
	BSF     	LCD_CNTL,E      ; toggle E for LCD
	nop
      BCF     	LCD_CNTL,E
      SWAPF   	CHAR, w
      ANDLW   	0xF0            ;Get lower nibble
	movwf		data_out
      call		serial
      BSF     	LCD_CNTL,E      ; toggle E for LCD
	nop
      BCF     	LCD_CNTL,E
      RETURN


;****************************************************************************************
;time delay to for interval setting of LCD
;****************************************************************************************
delay_interval
	MOVLW   	0x16
      MOVWF   	MSD             ; Use MSD and LSD Registers to Initilize LCD
      CLRF    	LSD             ;
LOOP  DECFSZ  	LSD, F          ; Delay time = MSD * ((3 * 256) + 3) * Tcy
      GOTO    	LOOP           ;
      DECFSZ  	MSD, F          ;
      GOTO    	LOOP           ;
	return

dlyus				;time delay to ensure 50% duty cycle clock timing
	nop
	nop
	nop
	nop
	return


;**********************************************************************************
; Display data routine
;**********************************************************************************
Display_data0
      MOVLW   	DISP_ON         ;
      CALL    	SEND_CMD        ;
      MOVLW   	CLR_DISP        ;
      CALL    	SEND_CMD        ;
      MOVLW   	ENTRY_INC       ;
      CALL    	SEND_CMD        ;
	MOVLW		ENTRY_INC       ;
      CALL    	SEND_CMD        ;
;
      movlw   	B'10000111'     ;Address DDRam first character, first line
      call    	SEND_CMD
	
;Demonstration of the use of a table to output a message
      movlw   	0               ;Table address of start of message
dispmsg0 
      movwf   	TEMP1           ;TEMP1 holds start of message address
      call    	Table0
      andlw   	0FFh            ;Check if at end of message (zero
      btfsc   	STATUS,Z        ;returned at end)
      goto		next
      call    	SEND_CHAR       ;Display character
      movf    	TEMP1,w         ;Point to next character
      addlw   	1
      goto    	dispmsg0
next
	movlw   	0xc2			;Address DDRam first character, first line
      call    	SEND_CMD
	movlw   	0               ;Table address of start of message
dispmsg1
      movwf   	TEMP1           ;TEMP1 holds start of message address
      call    	Table1
      andlw   	0FFh            ;Check if at end of message (zero
      btfsc   	STATUS,Z        ;returned at end)
      return
      call    	SEND_CHAR       ;Display character
      movf    	TEMP1,w         ;Point to next character
      addlw   	1
      goto    	dispmsg1


;****************************************************************************************
;Table of character will be display
;****************************************************************************************
Table0
      addwf   PCL, F          ;Jump to char pointed to in W reg
      retlw   'c'
      retlw   'h'
      retlw   'a'
      retlw   'n'
      retlw   'n'
      retlw   'e'
      retlw   'l'
      retlw   ' '
	retlw   '0'
      retlw   0  


Table1
      addwf   PCL, F          ;Jump to char pointed to in W reg
      retlw   'H'
      retlw   'e'
      retlw   'x'
      retlw   ' '
      retlw   'v'
      retlw   'a'
      retlw   'l'
      retlw   'u'
	retlw   'e'
      retlw   0  

;******************************************************************************
;communicate with ADC chip (MCP320X-chip1) subrotin.
;******************************************************************************
BEGIN_ADC1
	bsf	spi_port1,CS1
	movf	ch_LSB_temp,w	;move SPI setting value and LSB channel setting value
	movwf	data_out1
	call	BEGIN_SPI1
	movf	ch_MSB_temp,w	;MSB channel setting value
	movwf	data_out1
	call	BEGIN_SPI1
	movf	rxdata1,w		;move SPI received data (from rxdata1) to w register
	andlw	0x0f			;logical and MSB value (ADC data) 
	movwf	adcMSB_temp		;move MSB value to adcMSB_temp address
	movlw	0x00
	movwf	data_out1
	call	BEGIN_SPI1
	movf	rxdata1,w		;move SPI received data (from rxdata1) to w register
	bsf	spi_port1,CS1
	movwf	adcLSB_temp		;move LSB value (ADC data) to adcLSB_temp address
	return


;******************************************************************************
;Software SPI communication subrotin (ADC chip1)
;******************************************************************************
BEGIN_SPI1
	bsf	spi_port1,SCK1
	clrf	rxdata1
	movlw	h'08'
	movwf	bit_count1
	bcf	spi_port1, DO1
	bcf	spi_port1,CS1
rwspiloop1
	movf	data_out1,w
	andlw	0x80
	movwf	data_temp1
	bcf	spi_port1,SCK1		;start here, sck = 0
	btfss	data_temp1,7		;check bit 7 equ to 1?
	bcf	spi_port1,DO1		;no, DO = 0
	btfsc	data_temp1,7		;No, check bit 7 equ to 0?
	bsf	spi_port1,DO1		;no, DO = 1
	rlf	rxdata1,1
	btfsc	spi_port1,DI1		;no, DO = 0, check DI input signal equ o 0?
	incf	rxdata1,1		;no, increment 1 to rxdata address
	nop
	call	delay
	bsf	spi_port1,SCK1	;sck = 1
	rlf	data_out1,1
	call	delay
	decfsz	bit_count1,1
	goto	rwspiloop1
	bsf	spi_port1,SCK1
	return


;*****************************************************************************
;delay subrotin
;*****************************************************************************
delay
	movlw	0x08
	movwf	dlybyte
dlyloop
	decfsz	dlybyte,1
	goto		dlyloop
	return

;********************************************************************
;sub: process ADC chennal 0 to 7
;********************************************************************
precess_chennal
	movf	STORE_BIT0, w
	xorlw	0x01
	btfsc	STATUS, Z
	goto	ch_0
	movf	STORE_BIT0, w
	xorlw	0x02
	btfsc	STATUS, Z
	goto	ch_1
	movf	STORE_BIT0, w
	xorlw	0x03
	btfsc	STATUS, Z
	goto	ch_2
	movf	STORE_BIT0, w
	xorlw	0x04
	btfsc	STATUS, Z
	goto	ch_3
	movf	STORE_BIT0, w
	xorlw	0x05
	btfsc	STATUS, Z
	goto	ch_4
	movf	STORE_BIT0, w
	xorlw	0x06
	btfsc	STATUS, Z
	goto	ch_5
	movf	STORE_BIT0, w
	xorlw	0x07
	btfsc	STATUS, Z
	goto	ch_6
	movf	STORE_BIT0, w
	xorlw	0x08
	btfsc	STATUS, Z
	goto	ch_7
	return
ch_0					;channel 0 (ID_temp is b'00000001')
	movlw	0x06
	movwf	ch_LSB_temp
	movlw	0x00
	movwf	ch_MSB_temp	
	call	process_binary
	movlw   	B'10001111'     ;Address DDRam first character, first line
      call    	SEND_CMD
	movlw   	0x30
      call    	SEND_CHAR
	return
	
ch_1					;channel 1 (ID_temp is b'00000010')
	movlw	0x06
	movwf	ch_LSB_temp
	movlw	0x40
	movwf	ch_MSB_temp	
	call	process_binary
	movlw   	B'10001111'     ;Address DDRam first character, first line
      call    	SEND_CMD
	movlw   	0x31
      call    	SEND_CHAR
	return

ch_2					;channel 2 (ID_temp is b'00000100')
	movlw	0x06
	movwf	ch_LSB_temp
	movlw	0x80
	movwf	ch_MSB_temp	
	call	process_binary
	movlw   	B'10001111'     ;Address DDRam first character, first line
      call    	SEND_CMD
	movlw   	0x32
      call    	SEND_CHAR
	return

ch_3					;channel 3 (ID_temp is b'00001000')	
	movlw	0x06
	movwf	ch_LSB_temp
	movlw	0xc0
	movwf	ch_MSB_temp	
	call	process_binary
	movlw   	B'10001111'     ;Address DDRam first character, first line
      call    	SEND_CMD
	movlw   	0x33
      call    	SEND_CHAR
	return

ch_4					;channel 4 (ID_temp is b'00000011')
	movlw	0x07
	movwf	ch_LSB_temp
	movlw	0x00
	movwf	ch_MSB_temp	
	call	process_binary
	movlw   	B'10001111'     ;Address DDRam first character, first line
      call    	SEND_CMD
	movlw   	0x34
      call    	SEND_CHAR
	return

ch_5					;channel 5 (ID_temp is b'00000101')
	movlw	0x07
	movwf	ch_LSB_temp
	movlw	0x40
	movwf	ch_MSB_temp	
	call	process_binary
	movlw   	B'10001111'     ;Address DDRam first character, first line
      call    	SEND_CMD
	movlw   	0x35
      call    	SEND_CHAR
	return

ch_6					;channel 6 (ID_temp is b'00001001')
	movlw	0x07
	movwf	ch_LSB_temp
	movlw	0x80
	movwf	ch_MSB_temp	
	call	process_binary
	movlw   	B'10001111'     ;Address DDRam first character, first line
      call    	SEND_CMD
	movlw   	0x36
      call    	SEND_CHAR
	return

ch_7					;channel 6 (ID_temp is b'00001001')
	movlw	0x07
	movwf	ch_LSB_temp
	movlw	0xc0
	movwf	ch_MSB_temp	
	call	process_binary
	movlw   	B'10001111'     ;Address DDRam first character, first line
      call    	SEND_CMD
	movlw   	0x37
      call    	SEND_CHAR
	return

process_binary
	call	BEGIN_ADC1
	movf 	adcLSB_temp,w
	call	Binary_convert
	movf	ADC_LSB_H,w
	movwf	ADC_F_LSB
	movf	ADC_MSB_H,w
	movwf	ADC_S_LSB
	movf 	adcMSB_temp,w
	call	Binary_convert
	movf	ADC_LSB_H,w
	movwf	ADC_F_MSB
	movf	ADC_MSB_H,w
	movwf	ADC_S_MSB
	return


;******************************************************************
;Binary data conversion subrotin 
;******************************************************************
Binary_convert
	movwf	ADC_temp1
	swapf	ADC_temp1,w
	movwf	ADC_temp2
	btfsc	ADC_temp1,3
	goto 	testbit1_F
	goto	process_L_LSB
testbit1_F
	btfsc	ADC_temp1,1
	goto	process_H_LSB
	btfsc	ADC_temp1,2
	goto	process_H_LSB
process_L_LSB
	movlw	0x0f
	andwf	ADC_temp1,1
	movlw	0x30
	iorwf	ADC_temp1,w
	movwf	ADC_LSB_H
	goto	process_ADC_temp2
process_H_LSB
	movlw	0x0f
	andwf	ADC_temp1,w
	addlw	0x07
	movwf	ADC_temp1
	movlw	0x0f
	andwf	ADC_temp1,1
	movlw	0x40
	iorwf	ADC_temp1,w
	movwf	ADC_LSB_H
process_ADC_temp2
	btfsc	ADC_temp2,3
	goto 	testbit1_S
	goto	process_L_MSB
testbit1_S
	btfsc	ADC_temp2,1
	goto	process_H_MSB
	btfsc	ADC_temp2,2
	goto	process_H_MSB	
process_L_MSB
	movlw	0x0f
	andwf	ADC_temp2,1
	movlw	0x30
	iorwf	ADC_temp2,w
	movwf	ADC_MSB_H
	return
process_H_MSB
	movlw	0x0f
	andwf	ADC_temp2,w
	addlw	0x07
	movwf	ADC_temp2
	movlw	0x0f
	andwf	ADC_temp2,1
	movlw	0x40
	iorwf	ADC_temp2,w
	movwf	ADC_MSB_H
	return

    	end




